from LuluTest.action.element_actions import ElementActions
from LuluTest.action.page_actions import PageActions
from LuluTest.action.page_actions_decorator import PageActionsDecorator
from LuluTest.page_element_interface.IPageElement import load_driver


class Action(PageActionsDecorator, ElementActions):
    def __init__(self, driver_type=None, options=None):
        if driver_type is None:
            driver_type = 'Chrome'
        if options is None:
            options = ['headless']
        self.driver = load_driver(driver_type, options)
        self.action_map = None
        PageActionsDecorator.__init__(self, PageActions(self.driver))
        ElementActions.__init__(self, self.driver)



import unittest

from LuluTest.element import *


class TestElements(unittest.TestCase):
    def test_page_element(self):
        el = PageElement(('id', 'test'), 'test element')
        self.assertTupleEqual(el.locator, ('id', 'test'))
        self.assertEqual(el.name, 'test element')
        self.assertFalse(el.is_alert_element)
        self.assertTrue(el.is_page_element)

    def test_alert_element(self):
        el = AlertElement('test element')
        self.assertEqual(el.name, 'test element')
        self.assertFalse(el.is_page_element)
        self.assertTrue(el.is_alert_element)

    def test_locator(self):
        test_locator = Locator('id', 'test')
        self.assertEqual(test_locator.by, 'id')
        self.assertEqual(test_locator.value, 'test')



import unittest

from LuluTest import PageElement, Locator
from LuluTest.page import *


class TestPage(unittest.TestCase):
    page = Page('www.example.com')
    element = PageElement(Locator('id', 'test'), 'test')

    def test_basic_page(self):
        self.assertEqual(self.page.url, 'www.example.com')

    def test_init_with_elements(self):
        p = Page('www.example.com', [self.element])
        self.assertEqual(p.url, 'www.example.com')
        self.assertEqual(len(p.elements), 1)

    def test_add_element(self):
        self.assertEqual(len(self.page.elements), 0)
        self.page.elements.append(self.element)
        self.assertEqual(len(self.page.elements), 1)

    def test_get_element_from_page(self):
        p = Page('www.example.com', [self.element])
        element_by_index = p.elements[0]
        self.assertEqual(element_by_index, self.element)
        element_by_name = [elem for elem in p.elements if elem.name == 'test'][0]
        self.assertEqual(element_by_name, self.element)



from LuluTest.action import *
from LuluTest.page import *


class Do:
    def __init__(self, steps):
        for step in steps.each:
            DoStep(step)


class DoStep:
    def __init__(self, step):
        self.subject = step.subject
        self.map = None
        self.__resolve_subject()
        self.operation = self.map.get(step.operation)
        self.__do_operation(step)

    def __resolve_subject(self):
        if type(self.subject) == Page or self.subject is None:
            self.map = page_action_map
        elif isinstance(self.subject, BaseElement):
            self.map = element_action_map

    def __do_operation(self, step):
        if not step.subject:
            self.operation(step.action_object)
        elif step.data:
            self.operation(step.action_object, self.subject, step.data)
        else:
            self.operation(step.action_object, self.subject)



import unittest

from LuluTest import *


class SmokeTest(unittest.TestCase):
    def test_smoke_test(self):
        erik_whiting = Page('http://erikwhiting.com')
        link = PageElement(('Link Text', 'blog'), 'blog link')
        actions = Action()
        actions.go(erik_whiting)
        actions.click(link)
        url = actions.get_url()
        self.assertEqual(url, 'https://blog.erikwhiting.com/')

    def test_page_not_loaded_exception(self):
        actions = Action()
        self.assertRaises(PageNotLoadedError, actions.get_url)

    def test_write_and_click(self):
        page = Page('http://erikwhiting.com/newsOutlet')
        actions = Action()
        page.elements = [
            PageElement(("id", "sourceNews"), "input box"),
            PageElement(("id", "transmitter"), "button"),
            PageElement(("id", "en1"), "english div")
        ]
        actions.go(page)
        actions.input_text(page.get_element("input box"), "Hello")
        actions.click(page.get_element("button"))
        english_div = page.get_element("english div")
        english_text = actions.check_element_text(english_div, "Hello")
        self.assertTrue(english_text)
        actions.close()

    def test_step(self):
        erik_whiting = Page('http://erikwhiting.com')
        actions = Action()
        erik_whiting.elements = [
            PageElement(("Link Text", "blog"), "blog link")
        ]
        actions.go(erik_whiting)
        step1 = Step(actions, "click", erik_whiting.get_element("blog link"))
        DoStep(step1)
        url = actions.get_url()
        self.assertEqual(url, 'https://blog.erikwhiting.com/')
        actions.close()

    def test_steps(self):
        actions = Action()
        news_site = Page('http://erikwhiting.com/newsOutlet')
        news_site.elements = [
            PageElement(("id", "sourceNews"), "input box"),
            PageElement(("id", "transmitter"), "button"),
            PageElement(("id", "en1"), "english div")
        ]
        actions.go(news_site)
        steps = Steps(actions, [
            ('type', news_site.get_element('input box'), 'Hello'),
            ('click', news_site.get_element('button'))
        ])
        Do(steps)
        english_text = actions.check_element_text(news_site.get_element('english div'), "Hello")
        self.assertTrue(english_text)
        actions.close()



from .action import Action, PageActions, ElementActions
from .element import BaseElement, AlertElement, PageElement, Locator
from .lulu_exceptions import *
from .page import Page, page_factory
from .page_element_interface import IPageElement
from .step import Step, Steps, Do, DoStep



from LuluTest.element import BaseElement


class PageElement(BaseElement):
    def __init__(self, locator, name=''):
        super().__init__(name)
        self.locator = locator
        self.is_page_element = True



from typing import List, Dict

import yaml
import re
import json

from LuluTest.element import BaseElement, PageElement
from LuluTest.page import Page


def generate_pages(files) -> Dict[str, Page]:
    files = [files] if isinstance(files, str) else files
    pages = {}
    for file in files:
        page_name = re.split('\\\\|/|\.', file)[-2]
        pages[page_name] = generate_page(file)
    return pages


def generate_page(file) -> Page:
    file_type = file.split('.')[-1]
    file_type = file_type.lower()
    gen_function = eval('generate_page_from_{}'.format(file_type))
    page = gen_function(file)
    return page


def generate_page_from_yml(file) -> Page:
    with open(file) as f:
        page_data = yaml.load(f, Loader=yaml.FullLoader)['page']
    url = page_data['url']
    page = Page(url)

    elements_data = page_data['elements'] if 'elements' in page_data else []
    page.elements = generate_elements(elements_data)

    return page


def generate_page_from_json(file) -> Page:
    with open(file) as file:
        page_data = json.load(file)['page']
    url = page_data['url']
    page = Page(url)
    json_elements = page_data['elements']
    element_dict = {}
    for je in json_elements:
        locator_dict = {je['locator']: je['value']}
        name = je['name']
        element_dict[name] = locator_dict
    page.elements = generate_elements(element_dict)
    return page


def generate_elements(elements_data) -> List[BaseElement]:
    elements = []
    for element_key in elements_data.keys():
        element_data = elements_data[element_key]
        locator = list(element_data)[0]
        locator_val = element_data[locator]
        page_element = PageElement((locator, locator_val), element_key)
        elements.append(page_element)
    return elements



class Step:
    def __init__(self, action_object, operation, subject=None, data=None):
        self.action_object = action_object
        self.operation = operation
        self.subject = subject
        self.data = data
        self.result = None



import unittest

from LuluTest.page import Page
from LuluTest.element import PageElement
from LuluTest.action import Action


class TestBrowsers(unittest.TestCase):
    page = Page('http://erikwhiting.com/newsOutlet')
    page.elements = [
        PageElement(("id", "sourceNews"), "input box"),
        PageElement(("id", "transmitter"), "button"),
        PageElement(("id", "en1"), "english div")
    ]
    actions = None

    def _set_up(self, browser):
        self.actions = Action(browser)
        self.actions.go(self.page)

    def tearDown(self):
        self.actions.close()
        del self.actions

    def test_page_source_feature_chrome(self):
        self._set_up('Chrome')
        source = self.actions.get_page_source()
        self.assertIn('<body onload="defaultBreaking()">', source)

    def test_page_refresh_chrome(self):
        self._set_up('Chrome')
        self.actions.refresh()
        current_url = self.actions.get_url()
        self.assertEqual('http://erikwhiting.com/newsOutlet/', current_url)

    def test_page_change_url_chrome(self):
        self._set_up('Chrome')
        current_url = self.actions.get_url()
        self.assertEqual('http://erikwhiting.com/newsOutlet/', current_url)

        self.actions.go_to('https://github.com')
        current_url = self.actions.get_url()
        self.assertEqual('https://github.com/', current_url)

    def test_page_source_feature_firefox(self):
        self._set_up('Firefox')
        source = self.actions.get_page_source()
        self.assertIn('<body onload="defaultBreaking()">', source)

    def test_page_refresh_firefox(self):
        self._set_up('Firefox')
        self.actions.refresh()
        current_url = self.actions.get_url()
        self.assertEqual('http://erikwhiting.com/newsOutlet/', current_url)

    def test_page_change_url_firefox(self):
        self._set_up('Firefox')
        current_url = self.actions.get_url()
        self.assertEqual('http://erikwhiting.com/newsOutlet/', current_url)

        self.actions.go_to('https://github.com')
        current_url = self.actions.get_url()
        self.assertEqual('https://github.com/', current_url)


from selenium.webdriver.support import expected_conditions as ec
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.by import By

from LuluTest.page_element_interface import browser_factory


def load_element(driver, element):
    waiter = None
    if element.is_page_element:
        resolver = get_element_by_resolver(element)
        waiter = ec.visibility_of_element_located(locator=resolver)
    elif element.is_alert_element:
        waiter = ec.alert_is_present()
    return WebDriverWait(driver, 10).until(waiter)


def check_element_text(driver, element, text):
    resolver = get_element_by_resolver(element)
    waiter = ec.text_to_be_present_in_element(locator=resolver, text_=text)
    return WebDriverWait(driver, 10).until(waiter)


def load_driver(browser_type, options=None):
    if options is None:
        options = []
    if 'not headless' not in options:
        options = ['headless']

    return browser_factory.new(browser_type, options)


def get_element_by_resolver(element):
    indicator, value = element.locator
    converter = {
        "id": By.ID,
        "xpath": By.XPATH,
        "selector": By.CSS_SELECTOR,
        "class": By.CLASS_NAME,
        "link text": By.LINK_TEXT,
        "partial link": By.PARTIAL_LINK_TEXT,
        "name": By.NAME,
        "tag": By.TAG_NAME
    }
    by_indicator = converter.get(indicator.lower())
    return by_indicator, value
